module.exports = function(grunt) {
    'use strict';

    // Project configuration.
    grunt.initConfig({
        pkg: grunt.file.readJSON('package.json'),

        jshint: {
            lib: ['lib/**/*.js'],
            demo: ['demo/**/*.js', 'demo/**/*.json'],
            grunt: ['Gruntfile.js'],
            test: ['test/main.js', 'test/unit/*.js']
        },

        copy: {
            demoIndex: { src: 'demo/index.html', dest: 'bin/index.html' },
            demoAssets: {
                expand: true,
                cwd: 'demo/resources',
                src: '**',
                dest: 'bin/resources'
            }
        },

        qunit: {
            all: ['test/index.html']
        },

        browserify: {
            demo: {
                src: ['demo/main.js'],
                dest: 'bin/main.js',
                options: { debug: true }
            },

            test: {
                src: ['test/main.js'],
                dest: 'test/bin/main.debug.js',
                options: { debug: true }
            }
        },

        uglify: {
            release: {
                src: ['bin/main.js'],
                dest: 'bin/main.min.js',
                options: { report: 'gzip' }
            },
        },

        watch: {
            demo: {
                files: ['demo/**/*.js', 'lib/**/*.js'],
                tasks: ['build:demo']
            },


            test: {
                files: ['lib/**/*.js', 'test/unit/**/*.js', 'test/main.js'],
                tasks: ['build:test']
            },

            options: { livereload: 35729 }
        }
    });

    // plugins
    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-copy');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-contrib-qunit');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-browserify');

    // Misc tasks
    grunt.registerTask('lint', ['jshint']);
    grunt.registerTask('test', ['build:test', 'qunit']);

    // Builds
    grunt.registerTask('build', ['build:test', 'build:demo']);
    grunt.registerTask('build:test', [
      'lint',
      'browserify:test'
    ]);
    grunt.registerTask('build:demo', [
      'lint',
      'copy:demoIndex',
      'copy:demoAssets',
      'browserify:demo'
    ]);
    grunt.registerTask('release', [
      'build:demo',
      'uglify:release'
    ]);

    grunt.registerTask('default', ['build']);

};
