# CGame

Low level, high-performance Javascript + canvas cross-platform 2D game engine.

Create killer 2D games in Javascript for the web, PC, Mac, Linux, Android, and
iPhone. All with the same codebase.

Buttery-smooth 60 FPS performance with a highly-optimized core.
Entity-Component-Systems architecture goodness for highly decoupled and
flexible game development.

Make some magic.

## Current Features

- Entity / Component / System architecture core
- Generic object pooling
- 2D Canvas renderer with static sprites
- Basic gamepad, mouse, keyboard, and multi-touch input system
- `Vec2`, `Entity`, `Component`, `Canvas` and other core types pooled
  comprehensively across engine
- High-performance quad-tree for maintaining a spatial index on thousands of
  entities in real-time
- Basic resource management via JSON files with namespacing / id support for
  references
- Sprites as a resource
- Assemblies as a resource

## Future Features

Roughly ordered by priority:

- Scene / Layer support for handling menus, game state
- Maps as a resource
- Abstracted persistence layer for storing game saves via e.g. local storage
- Abstracted input layer for handling cross-platform controls
- Sound support with distance / volume mixing
- Sound as a resource
- Dynamic 2D lighting with shadow / ray casting support
- High-performance particle field system
- Animated sprites via sprite sheets

## Stock Components

**CGame** comes with some general-purpose components.

### Attached

Attaches an entity onto another entity. Components that may have different
behavior when attached should have helper methods for ensuring the calculated
values are correct.

For example, the `Position` component has `Position.getAbsolutePosition()` to
resolve the actual world-coordinates recursively.

**Examples**: Turrets mounted on a ship, pieces of a large entity that function
seperately, cameras attached to players.

### Spatial

Gives an entity the ability to be part of the spatial indexing. This is
maintained as a QuadTree that allows you to get "roughly" coincident entities,
which can then be further inspected for collisions, considered in nearby
algorithms, etc.

**Examples**: Entities that have to be checked for collisions, entities that
are used in location-specific areas (such as lighting things within a certain
range, checking enemies in a certain range)...

### Steering

An entity can contain a "steering" vector that indicates the desired heading.
The magnitude (will usually) represent the desired velocity vector.

Typically, this will be set by a player input or some AI system, while a
Locomotion system will take the steering vector, acount for entity-specific
constraints (such as max speed, turning ability, etc), and impart some
acceleration in the `Newtonian` componet.

### Camera

Give an entity the ability to be used as a camera for rendering to the screen
or texture.

### Collision

### Expiry

An entity with an `Expiry` component will be removed after a certain amount of
time.

### Position

Any entity that requires an position vector and rotation value have a
`Position` component.

### Sprite

## Stock Systems

## Testing

This is some test coverage for a few of the low-level data structures used in
**CGame**.

Testing requires [node/npm](http://nodejs.org) and
[grunt-cli](https://github.com/gruntjs/grunt-cli) to be installed on your
system.

To install all the dev dependencies and run the test target:

```
npm install
grunt test
```

## License
Copyright 2013 Brandon Valosek

**CGame** is released under the MIT license.

