var __extends        = require('../lib/util/extends.js');
var Game             = require('../lib/Game.js');
var Scene            = require('../lib/Scene.js');
var Vec2             = require('../lib/util/Vec2.js');
var ResourceManager  = require('../lib/resources/ResourceManager.js');
var EntityLayer      = require('../lib/layers/EntityLayer.js');
var FillLayer        = require('../lib/layers/FillLayer.js');
var DebugLayer       = require('../lib/layers/DebugLayer.js');
var Position         = require('../lib/ecs/components/Position.js');
var Newtonian        = require('../lib/ecs/components/Newtonian.js');
var Camera           = require('../lib/ecs/components/Camera.js');
var Attached         = require('../lib/ecs/components/Attached.js');
var Sprite           = require('../lib/ecs/components/Sprite.js');
var Expiry           = require('../lib/ecs/components/Expiry.js');
var Spatial          = require('../lib/ecs/components/Spatial.js');
var Elastic          = require('../lib/ecs/components/Elastic.js');
var Collision        = require('../lib/ecs/components/Collision.js');
var Mover            = require('../lib/ecs/systems/Mover.js');
var Physics          = require('../lib/ecs/systems/Physics.js');
var Renderer         = require('../lib/ecs/systems/Renderer.js');
var Expiry           = require('../lib/ecs/systems/Expiry.js');
var TreeMaintainer   = require('../lib/ecs/systems/TreeMaintainer.js');
var Collider         = require('../lib/ecs/systems/Collider.js');

var PlayerControlled = require('./components/PlayerControlled.js');
var Gun              = require('./components/Gun.js');
var Targeting        = require('./components/Targeting.js');
var PlayerInput      = require('./systems/PlayerInput.js');
var Targeter         = require('./systems/Targeter.js');

// Setup game
global.game = new Game().start();
global.rm = new ResourceManager();
rm.basePath = 'resources/';

// Register resources
rm.registerResource(require('./sprites/ship.json'));
rm.registerResource(require('./sprites/gun.json'));

var asteroidSprites = [];
asteroidSprites.push(rm.registerResource(require('./sprites/asteroid1.json')));
asteroidSprites.push(rm.registerResource(require('./sprites/asteroid2.json')));
asteroidSprites.push(rm.registerResource(require('./sprites/asteroid3.json')));

rm.registerResource(require('./assemblies/playerShip.json'));
rm.registerResource(require('./assemblies/camera.json'));
rm.registerResource(require('./assemblies/turret.json'));

rm.resolveReferences();
rm.downloadResources(function() {
  var x;

  console.log(rm.resources);

  // Setup game ES layer w/ our systems
  layer = new EntityLayer();
  layer.systemManager.addSystem(Expiry);
  layer.systemManager.addSystem(PlayerInput);
  layer.systemManager.addSystem(Targeter);
  layer.systemManager.addSystem(Physics);
  layer.systemManager.addSystem(Collider);
  layer.systemManager.addSystem(Mover);
  var treeSystem = layer.systemManager.addSystem(TreeMaintainer);
  layer.systemManager.addSystem(Renderer);

  layer.entityManager.registerComponent(Position);
  layer.entityManager.registerComponent(Newtonian);
  layer.entityManager.registerComponent(Camera);
  layer.entityManager.registerComponent(Attached);
  layer.entityManager.registerComponent(Sprite);
  layer.entityManager.registerComponent(Expiry);
  layer.entityManager.registerComponent(Spatial);
  layer.entityManager.registerComponent(Collision);
  layer.entityManager.registerComponent(PlayerControlled);
  layer.entityManager.registerComponent(Gun);
  layer.entityManager.registerComponent(Targeting);

  // Create scene
  scene = new Scene();
  game.addScene(scene);
  layer.zIndex = 1;
  scene.addLayer(layer);

  // Setup back layer
  var background = new FillLayer();
  background.zIndex = -10;
  background.color = '#080808';
  scene.addLayer(background);

  // Debug layer
  var debug = new DebugLayer();
  debug.zIndex = 100;
  debug.entityManager = layer.entityManager;
  debug.treeSystem = treeSystem;
  debug.tree = treeSystem.tree;
  scene.addLayer(debug);


  var range  = 80000;
  var tote   = 2000;
  var vrange = 3;
  for (i = 0; i < tote; i++) {
    var a = layer.entityManager.addNewEntity();
    a.addComponent(Newtonian);
    a.addComponent(Position);
    a.addComponent(Spatial);
    a.Spatial.size = 100;
    var size = Math.random()*2 + 0.25;
    size *= Math.max(0.2, 15 * i/tote);
    a.Position.scale.set(size, size);
    a.addComponent(Sprite);
    a.Sprite.resource = asteroidSprites[Math.floor(Math.random()*2)];
    a.Newtonian.velocity.set(Math.random()*vrange-vrange/2, Math.random()*vrange-vrange/2);
    a.Newtonian.radialSpeed = Math.random()*1-0.5;
    a.Position.rotation = Math.random()*Math.PI*2;
    var mult = Math.max(3.75 * i / tote, 1);
    x = (i / tote) * range + 600;
    var y = (i / tote) * range + 600;
    a.Position.location.set(x, y);
    a.Position.location.smult(mult);
    a.Position.location.rotate(Math.PI*2*Math.random());
    a.Position.height = Math.random()*500-250;
  }

  // Add player
  global.player = layer.entityManager.addNewAssembly(
  rm.resources.assemblies.playerShip);
  player.addTag('player');

  // Add guns
  for (x = 0; x < 3; x++) {
    var gun = layer.entityManager.addNewAssembly(
    rm.resources.assemblies.turret);
    gun.Attached.parentEntity = player;

    switch (x) {
      case 0:
        gun.Position.location.set(100, -130);
        break;
      case 1:
        gun.Position.location.set(100, 120);
        break;
      case 2:
        gun.Position.location.set(130, -5);
        break;
    }
  }



  // Add a camera
  var camera = layer.entityManager.addNewAssembly(
  rm.resources.assemblies.camera);
  camera.Attached.parentEntity = player;
  camera.addTag('mainCamera');
  camera.Camera.canvas = Game.instance.screen;
  camera.Camera.scale = 1;


});
