var Targeting = require('../components/Targeting.js');
var Vec2      = require('../../lib/util/Vec2.js');
var Position  = require('../../lib/ecs/components/Position.js');

module.exports = (function() {

    Targeter.componentTypes = [Targeting];

    // System that executes the stragegy for any entities that are targeting
    // something
    function Targeter()
    {

    }

    Targeter.prototype.update = function(dt, time)
    {
        var entities = this.entityManager
            .getEntitiesByComponents(Targeter.componentTypes);

        for (var n = 0; n < entities.length; n++) {
            var entity = entities[n];
            var info   = entity.Targeting;
            var strat  = entity.Targeting.targetingStrategy;

            if (!strat) continue;

            Targeter[strat](entity, info);
        }
    };

    // Always accelerate towards a target
    Targeter.accelerateTowards = function(entity, info)
    {
        var targetEntity = info.targetEntity;
        if (!targetEntity)
            return;

        // absolute Entity to target vector
        var eToT  = targetEntity.Position.location.pcopy()
            .sub(Position.getAbsoluteLocation(entity));

        entity.Newtonian.acceleration.assign(eToT);

        Vec2.release(eToT);
    };

    // Rotate towards an entity
    Targeter.trackEntity = function(entity, info)
    {
        var targetEntity = info.targetEntity;
        if (!targetEntity)
            return;

        info.targetPosition = Position.getAbsoluteLocation(targetEntity);

        Targeter.track(entity, info);
    };

    Targeter.trackAngle = function(entity, info, angle)
    {
        angle = angle || info.targetAngle;

        // The relative angle we need on this entity to get the desired
        // absolate angle
        var targetAngle = Position.toRelativeRotation(entity, angle);

        // Angle vectors that we want and need
        var rv = Vec2.pfromAngle(entity.Position.rotation);
        var trv = Vec2.pfromAngle(targetAngle);
        var det = rv.det(trv);

        var diff = Math.abs(trv.angleBetween(rv));
        var maxRad = entity.Newtonian.maxRadialSpeed;

        var per = maxRad / 60;

        if (diff > per) {
            entity.Newtonian.radialSpeed = det > 0 ? maxRad : -maxRad;
        } else {
            entity.Newtonian.radialSpeed = 0;
            entity.Position.rotation = targetAngle;
        }

        Vec2.release(rv);
        Vec2.release(trv);
    };

    // Simply rotate such that we are always facing towards a target
    Targeter.track = function(entity, info)
    {
        var targetPos = info.targetPosition;
        if (!targetPos)
            return;

        // absolute Entity to target vector
        var eToT  = targetPos.pcopy()
            .sub(Position.getAbsoluteLocation(entity));
        var angle = eToT.angle();


        Targeter.trackAngle(entity, info, angle);

        Vec2.release(eToT);
    };

    return Targeter;

})();
