var Vec2      = require('../../lib/geometry/Vec2');
var Gun       = require('../components/Gun');
var Game      = require('../../lib/Game');
var Position  = require('../../lib/components/Position');
var Newtonian = require('../../lib/components/Newtonian');
var Sprite    = require('../../lib/components/Sprite');
var Spatial   = require('../../lib/components/Spatial');
var Collision = require('../../lib/components/Collision');
var Expiry    = require('../../lib/components/Expiry');
var Bullet    = require('../components/Bullet');
var __extends = require('../../lib/util/extends');
var System    = require('../../lib/systems/System');


module.exports = (function(_super) {
    __extends(Weapons, _super);

    Weapons.gunFilter = [Gun];

    function Weapons()
    {
        _super.apply(this, arguments);

        // Create resources
        var game = Game.instance;
        this.listenTo(Collision.coincident, [Bullet], this.bulletCollision);
    }

    Weapons.prototype.bulletCollision = function(bullet, others)
    {
        for (var n = 0; n < others.length; n++) {
            var other = others[n];
            if (other === bullet.Bullet.owner)
                return;
            if (other.Collision &&
                other.Collision.ignoreFlag === other.Collision.ignoreFlag)
                    return;
            other.remove();
        }

        bullet.remove();
    };


    Weapons.prototype.update = function(dt, time)
    {
        var entities = this.entityManager.getEntitiesByComponents(
            Weapons.gunFilter);

        var game = Game.instance;

        for (var n = 0; n < entities.length; n++) {
            var entity = entities[n];

            if (!entity.Gun.firing) continue;
            var G = entity.Gun;

            var canFire = time > G.lastFire + 1000/G.fireRate;
            if (canFire && G.firing) {
                G.lastFire = time;

                var bullet = this.entityManager.addNewEntity();
                bullet.addComponent(Expiry);
                bullet.Expiry.expireAt = time + 3000;

                bullet.addComponent(Bullet);
                bullet.Bullet.owner = entity.Attached.parentEntity;

                bullet.addComponent(Position);
                bullet.Position.location.assign(
                    Position.getAbsoluteLocation(entity));
                bullet.Position.rotation = Position.getAbsoluteRotation(entity);

                bullet.addComponent(Sprite);
                bullet.Sprite.image =
                    game.resources.getResource('bulletSprite').canvas._canvas;
                bullet.Sprite.offset =
                    game.resources.getResource('bulletSprite').offset;

                bullet.addComponent(Newtonian);
                var rv = Vec2.pfromAngle(bullet.Position.rotation)
                    .normalize(G.speed);
                bullet.Newtonian.velocity
                    .assign(Newtonian.getAbsoluteVelocity(entity))
                    .add(rv);
                Vec2.release(rv);

                bullet.addComponent(Spatial);
                bullet.addComponent(Collision);
                bullet.Collision.ignoreFlag = 'player';
                bullet.Spatial.size = 10;
            }
        }
    };

    return Weapons;

})(System);
