var Pool       = require('../../lib/util/Pool');
var LinkedList = require('../../lib/util/LinkedList');

QUnit.module('Linked list');

test('basics', function() {

    var a = {};
    var b = {};
    var c = {};

    var list = new LinkedList();

    var an = list.push(a);
    var bn = list.push(b);
    var cn = list.push(c);

    strictEqual(list.count, 3, 'count');
    strictEqual(list.first, an, 'first node');
    strictEqual(list.last, cn, 'last node');

    strictEqual(an.next, bn, 'next');
    strictEqual(cn.prev, bn, 'prev');
    strictEqual(bn.next, cn, 'mid next');
    strictEqual(bn.prev, an, 'mid prev');

});

test('removal', function() {

    var a = {};
    var b = {};
    var c = {};

    var list = new LinkedList();

    var an = list.push(a);
    var bn = list.push(b);
    var cn = list.push(c);

    list.removeNode(bn);

    strictEqual(list.count, 2);
    strictEqual(an.next, cn, 'a joined');
    strictEqual(cn.prev, an, 'b joined');

    var x = {};
    var xn = list.push(x);
    strictEqual(xn, bn, 'recycled');
    strictEqual(cn.next, xn, 'added');
    strictEqual(list.last, xn, 'last');


});

test('clear', function() {

    var a = {};
    var b = {};
    var c = {};

    var list = new LinkedList();

    var an = list.push(a);
    var bn = list.push(b);
    var cn = list.push(c);

    list.clear();
    strictEqual(list.count, 0, 'no count');
    strictEqual(list.first, null, ' no first');
    strictEqual(list.last, null, 'no last');

    var xn = list.push(123);
    strictEqual(xn, cn, 'recycled');

});
